@@ -1,12 +1,11 @@
-/*
- *  File ergm/src/edgetree.c
- *  Part of the statnet package, http://statnet.org
+/*  File src/edgetree.c in package ergm, part of the Statnet suite
+ *  of packages for network analysis, http://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
- *  open source, and has the attribution requirements (GPL Section 7) in
- *    http://statnet.org/attribution
+ *  open source, and has the attribution requirements (GPL Section 7) at
+ *  http://statnet.org/attribution
  *
- *  Copyright 2012 the statnet development team
+ *  Copyright 2003-2013 Statnet Commons
  */
 #include "edgetree.h"
 
@@ -39,14 +38,13 @@ Network NetworkInitialize(Vertex *tails, Vertex *heads, Edge nedges,
   nw.inedges = (TreeNode *) calloc(nw.maxedges,sizeof(TreeNode));
   nw.outedges = (TreeNode *) calloc(nw.maxedges,sizeof(TreeNode));
 
-  GetRNGstate();  /* R function enabling uniform RNG */
+  /* GetRNGstate(); R function enabling uniform RNG */
 
   if(lasttoggle_flag){
-    unsigned int ndyads = bipartite? (nnodes-bipartite)*bipartite : (directed_flag? nnodes*(nnodes-1) : (nnodes*(nnodes-1))/2);
-    nw.duration_info.MCMCtimer=time;
-    nw.duration_info.lasttoggle = (int *) calloc(ndyads, sizeof(int));
+    nw.duration_info.time=time;
+    nw.duration_info.lasttoggle = (int *) calloc(DYADCOUNT(nnodes, bipartite, directed_flag), sizeof(int));
     if(lasttoggle)
-      memcpy(nw.duration_info.lasttoggle, lasttoggle, ndyads * sizeof(int));
+      memcpy(nw.duration_info.lasttoggle, lasttoggle, DYADCOUNT(nnodes, bipartite, directed_flag) * sizeof(int));
   }
   else nw.duration_info.lasttoggle = NULL;
 
@@ -65,7 +63,7 @@ Network NetworkInitialize(Vertex *tails, Vertex *heads, Edge nedges,
     else 
       AddEdgeToTrees(tail,head,&nw);
   }
-  PutRNGstate();
+  /* PutRNGstate(); */
   return nw;
 }
 
@@ -131,16 +129,16 @@ Network *NetworkCopy(Network *dest, Network *src){
   memcpy(dest->outedges, src->outedges, maxedges*sizeof(TreeNode));
 
   int directed_flag = dest->directed_flag = src->directed_flag;
+  Vertex bipartite = dest->bipartite = src->bipartite;
 
   if(src->duration_info.lasttoggle){
-    dest->duration_info.MCMCtimer=src->duration_info.MCMCtimer;
-    dest->duration_info.lasttoggle = (int *) calloc(directed_flag? nnodes*(nnodes-1) : (nnodes*(nnodes-1))/2, sizeof(int));
-    memcpy(dest->duration_info.lasttoggle, src->duration_info.lasttoggle,(directed_flag? nnodes*(nnodes-1) : (nnodes*(nnodes-1))/2) * sizeof(int));
+    dest->duration_info.time=src->duration_info.time;
+    dest->duration_info.lasttoggle = (int *) calloc(DYADCOUNT(nnodes, bipartite, directed_flag), sizeof(int));
+    memcpy(dest->duration_info.lasttoggle, src->duration_info.lasttoggle,DYADCOUNT(nnodes, bipartite, directed_flag) * sizeof(int));
   }
   else dest->duration_info.lasttoggle = NULL;
 
   dest->nedges = src->nedges;
-  dest->bipartite = src->bipartite;
 
   return dest;
 }
@@ -300,7 +298,7 @@ int ToggleEdgeWithTimestamp(Vertex tail, Vertex head, Network *nwp){
       else
 	k = (head-1)*(head-2)/2 + tail - 1;    
     }
-    nwp->duration_info.lasttoggle[k] = nwp->duration_info.MCMCtimer;
+    nwp->duration_info.lasttoggle[k] = nwp->duration_info.time;
   }
   
   if (AddEdgeToTrees(tail,head,nwp))
@@ -345,7 +343,7 @@ int ElapsedTime(Vertex tail, Vertex head, Network *nwp){
       else
 	k = (head-1)*(head-2)/2 + tail - 1;    
     }
-    return nwp->duration_info.MCMCtimer - nwp->duration_info.lasttoggle[k];
+    return nwp->duration_info.time - nwp->duration_info.lasttoggle[k];
   }
   else return 0; 
   /* Should maybe return an error code of some sort, since 0 elapsed time
@@ -379,7 +377,7 @@ void TouchEdge(Vertex tail, Vertex head, Network *nwp){
       else
 	k = (head-1)*(head-2)/2 + tail - 1;    
     }
-    nwp->duration_info.lasttoggle[k] = nwp->duration_info.MCMCtimer;
+    nwp->duration_info.lasttoggle[k] = nwp->duration_info.time;
   }
 }
 
@@ -617,12 +615,19 @@ int FindithEdge (Vertex *tail, Vertex *head, Edge i, Network *nwp) {
   Vertex taili=1;
   Edge e;
 
+  /* TODO: This could be speeded up by a factor of 3 or more by starting
+     the search from the tail n rather than tail 1 if i > ndyads/2. */
+
   if (i > nwp->nedges || i<=0)
     return 0;
   while (i > nwp->outdegree[taili]) {
     i -= nwp->outdegree[taili];
     taili++;
   }
+
+  /* TODO: This could be speeded up by a factor of 3 or more by starting
+     the search from the tree maximum rather than minimum (left over) i > outdegree[taili]. */
+
   e=EdgetreeMinimum(nwp->outedges,taili);
   while (i-- > 1) {
     e=EdgetreeSuccessor(nwp->outedges, e);
@@ -675,7 +680,7 @@ int GetRandEdge(Vertex *tail, Vertex *head, Network *nwp) {
 }
 
 /*****************
-  int FindithnonEdge
+  int FindithNonedge
 
   Find the ith nonedge in the Network *nwp and
   update the values of tail and head appropriately.  Return
@@ -688,11 +693,99 @@ int GetRandEdge(Vertex *tail, Vertex *head, Network *nwp) {
    be needed. */      
   /* *** but if it is needed, don't forget,  tail -> head */
 
-/* int FindithnonEdge (Vertex *tail, Vertex *head, Edge i, Network *nwp) {
-} */
+int FindithNonedge (Vertex *tail, Vertex *head, Edge i, Network *nwp) {
+  Vertex taili=1;
+  Edge e;
+  Edge ndyads = DYADCOUNT(nwp->nnodes, nwp->bipartite, nwp->directed_flag);
+  
+  // If the index is too high or too low, exit immediately.
+  if (i > ndyads - nwp->nedges || i<=0)
+    return 0;
 
+  /* TODO: This could be speeded up by a factor of 3 or more by starting
+     the search from the tail n rather than tail 1 if i > ndyads/2. */
 
+  Vertex nnt;
+  while (i > (nnt = nwp->nnodes - (nwp->bipartite ? nwp->bipartite : (nwp->directed_flag?1:taili))
+	      - nwp->outdegree[taili])) {   // nnt is the number of nonties incident on taili. Note that when network is undirected, tail<head.
+    i -= nnt;
+    taili++;
+  }
 
+  // Now, our tail is taili.
+
+  /* TODO: This could be speeded up by a factor of 3 or more by starting
+     the search from the tree maximum rather than minimum (left over) i > outdegree[taili]. */
+
+  // If taili 1, then head cannot be 1. If undirected, the smallest it can be is taili+1. If bipartite, the smallest it can be is nwp->bipartite+1.
+  Vertex lhead = (
+		  nwp->bipartite ? 
+		  nwp->bipartite :
+		  (nwp->directed_flag ?
+		   taili==1 : taili)
+		  );
+  e = EdgetreeMinimum(nwp->outedges,taili);
+  Vertex rhead = nwp->outedges[e].value;
+  // Note that rhead-lhead-1-(lhead<taili && taili<rhead) is the number of nonties between two successive ties.
+  // the -(lhead<taili && taili<rhead) is because (taili,taili) is not a valid nontie and must be skipped.
+  // Note that if taili is an isolate, rhead will be 0.
+  while (rhead && i > rhead-lhead-1-(lhead<taili && taili<rhead)) {
+    i -= rhead-lhead-1-(lhead<taili && taili<rhead);
+    lhead = rhead;
+    e = EdgetreeSuccessor(nwp->outedges, e);
+    // If rhead was the highest-indexed head, then e is now 0.
+    if(e) rhead = nwp->outedges[e].value;
+    else break; // Note that we don't actually need rhead in the final step.
+  }
+
+  // Now, the head we are looking for is (left over) i after lhead.
+
+  *tail = taili;
+  *head = lhead + i + (nwp->directed_flag && lhead+i>=taili); // Skip over the (taili,taili) dyad, if the network is directed.
+
+  return 1;
+}
+
+/*****************
+  int GetRandNonedge
+
+  Select an non-edge in the Network *nwp at random and update the values
+  of tail and head appropriately. Return 1 if successful, 0 otherwise.
+******************/
+
+/* *** don't forget tail->head, so this function now accepts tail before head */
+
+int GetRandNonedge(Vertex *tail, Vertex *head, Network *nwp) {
+  Edge ndyads = DYADCOUNT(nwp->nnodes, nwp->bipartite, nwp->directed_flag);
+  if(ndyads-nwp->nedges==0) return(0);
+
+  /* There are two ways to get a random nonedge: 1) keep trying dyads
+     at random until you find one that's not an edge or 2) generate i
+     at random and find ith nonedge. Method 1 works better in sparse
+     networks, while Method 2, which runs in deterministic time, works
+     better in dense networks.
+
+     The expected number of attempts for Method 1 is 1/(1-e/d) =
+     d/(d-e), where e is the number of edges and d is the number of
+     dyads.
+  */
+
+  // FIXME: The constant maxEattempts needs to be tuned.
+  const unsigned int maxEattempts=10;
+  unsigned int Eattempts = ndyads/(ndyads-nwp->nedges);
+  Edge rane;
+  
+  if(Eattempts>maxEattempts){
+    // If the network is too dense, use the deterministic-time method:
+    rane=1 + unif_rand() * (ndyads-nwp->nedges);
+    FindithNonedge(tail, head, rane, nwp);
+  }else{
+    do{
+      GetRandDyad(tail, head, nwp);
+    }while(EdgetreeSearch(*tail, *head, nwp->outedges));
+  }
+  return 1;
+}
 
 /* *** don't forget, edges are now given by tails -> heads, and as
        such, the function definitions now require tails to be passed
